
# interbase_adaptor.rb
# author: Kazuhiro Yoshida <moriq@moriq.com>

require 'active_record/connection_adapters/abstract_adapter'
require 'parsedate'

module ActiveRecord
  class Base
    def self.interbase_connection(config) # :nodoc:
      require 'interbase'
      symbolize_strings_in_hash(config)
      if config.has_key?(:database)
        database = config[:database]
      else
        raise ArgumentError, "No database specified. Missing argument: database."
      end
      username = config[:username] ? config[:username].to_s : 'SYSDBA'
      password = config[:password] ? config[:username].to_s : 'masterkey'
      charset  = config[:charset]
      ConnectionAdapters::InterBaseAdapter.new(
        InterBase::connect(database, username, password, charset), logger
      )
    end
  end
  
  module ConnectionAdapters
    class InterBaseAdapter < AbstractAdapter # :nodoc:
      def select_all(sql, name = nil)
        select(sql, name)
      end
      
      def select_one(sql, name = nil)
        result = select(sql, name)
        result.nil? ? nil : result.first
      end

      def columns(table_name, name = nil)
        table_structure(table_name, name).inject([]) do |columns, (field_name, type, default, length)|
          columns << Column.new(field_name, default, type)
          columns
        end
      end

      def execute(sql, name = nil)
        log(sql, name, @connection) { |connection| connection.execute(sql) }
      end

      def insert(sql, name = nil, pk = nil, id_value = nil)
        execute(sql, name = nil)
        table = sql.split(" ", 4)[2]
        id_value || last_insert_id(table, pk)
      end

      alias_method :update, :execute
      alias_method :delete, :execute

      def begin_db_transaction()
        # auto-start
      end

      def commit_db_transaction
        @connection.commit
      end

      def rollback_db_transaction
        @connection.rollback
      end

      def quote(value, column = nil)
        case value
        when String
          if column && column.type == :date
            value, timestr = value.split(/ /, 2)
          end
          "'#{quote_string(value)}'" # ' (for ruby-mode)
        when NilClass
          "NULL"
        when TrueClass
          (column && column.type == :boolean) ? "'t'" : "1"
        when FalseClass
          (column && column.type == :boolean) ? "'f'" : "0"
        when Float, Fixnum, Bignum
          "'#{value.to_s}'"
        when Time, Date, DateTime
          format =
            if column && column.type == :date
            then "%Y-%m-%d"
            else "%Y-%m-%d %H:%M:%S"
            end
            "'#{value.strftime(format)}'"
        else 
          "'#{quote_string(value.to_yaml)}'"
        end
      end

      def quote_string(s)
        s.gsub(/'/, "''") # ' (for ruby-mode)
      end

      def quote_column_name(name)
        %Q("#{name.upcase}")
      end

    private
      def select(sql, name = nil)
        curs = nil
        log(sql, name, @connection) { |connection| curs = connection.execute(sql) }
        results = curs.fetchall
        rows = []
        if results.length > 0
          description = curs.description
          results.each do |row|
            hashed_row = {}
            row.each_with_index do |cel, cel_index|
              case cel
              when Time
                cel = cel.strftime('%Y-%m-%d %H:%M:%S')
              end
              hashed_row[description[cel_index][0].downcase] = cel
            end
            rows << hashed_row
          end
        end
        rows
      end

      def last_insert_id(table, column = "id")
        generator_name = "gen_#{table}_#{column || 'id'}"
sql = "SELECT gen_id(#{generator_name},0) from rdb$database"
        ret = nil
        log(sql, nil, @connection) { |connection| ret = connection.execute(sql).fetch[0] }
        ret
      end

      def table_structure(table_name, name = nil)
        sql = "SELECT RFR.RDB$FIELD_NAME, RFR.RDB$DEFAULT_SOURCE, RDB$FIELD_LENGTH, RDB$FIELD_TYPE "
        sql << "  FROM RDB$FIELDS FLD "
        sql << "  JOIN RDB$RELATION_FIELDS RFR "
        sql << "    ON RFR.RDB$FIELD_SOURCE = FLD.RDB$FIELD_NAME "
        sql << " WHERE RFR.RDB$RELATION_NAME = '#{table_name.upcase}';"
        column_defaults = nil
        log(sql, name, @connection) { |connection| column_defaults = connection.execute(sql).fetchall }
        ret = column_defaults.collect do |field_name, default_source, field_length, field_type|
          field_name = field_name.sub(/ +$/, '').downcase
          type = type_as_string(field_type, field_length)
          default = default_value(default_source)
          [ field_name, type, default, field_length ]
        end
        ret
      end

      require 'active_record/vendor/interbase/blrutils'

      def type_as_string(field_type, field_length)
        type = InterBase.blr_type_name(field_type)
        case type
        when /CHAR/
          if field_length.nil?
            size = "" 
        # elsif field_length == 1
        #   type = "boolean"
        #   size = ""
          else
            size = "(#{field_length})"
          end
        else
          size = ""
        end
        type + size
      end

      def default_value(value)
        if value =~ /^default +(.*)/i
        then $1
        else value
        end
      end
    end
  end
end
