SET SQL DIALECT 3; 
 
/* CREATE DATABASE 'e:\ACTIVERECORD_UNITTEST.GDB' DEFAULT CHARACTER SET NONE */


CREATE GENERATOR GEN_ACCOUNTS_ID;
CREATE GENERATOR GEN_AUTO_ID_TESTS_AUTO_ID;
CREATE GENERATOR GEN_BOOLEANTESTS_ID;
CREATE GENERATOR GEN_COLNAMETESTS_ID;
CREATE GENERATOR GEN_COMPANIES_ID;
CREATE GENERATOR GEN_TOPICS_ID;

/* Domain definitions */
CREATE DOMAIN BOOLEAN AS SMALLINT
         default 0
         check (value in (0,1));
CREATE DOMAIN SERIAL AS INTEGER NOT NULL;
CREATE DOMAIN TEXT AS BLOB SUB_TYPE TEXT SEGMENT SIZE 80;

/* Table: ACCOUNTS, Owner: SYSDBA */
CREATE TABLE ACCOUNTS (ID INTEGER NOT NULL,
        FIRM_ID INTEGER,
        CREDIT_LIMIT INTEGER,
PRIMARY KEY (ID));

/* Table: AUTO_ID_TESTS, Owner: SYSDBA */
CREATE TABLE AUTO_ID_TESTS (AUTO_ID SERIAL,
        "VALUE" INTEGER,
PRIMARY KEY (AUTO_ID));

/* Table: BOOLEANTESTS, Owner: SYSDBA */
CREATE TABLE BOOLEANTESTS (ID SERIAL,
        "VALUE" BOOLEAN,
PRIMARY KEY (ID));

/* Table: COLNAMETESTS, Owner: SYSDBA */
CREATE TABLE COLNAMETESTS (ID SERIAL,
        "REFERENCES" INTEGER NOT NULL);

/* Table: COMPANIES, Owner: SYSDBA */
CREATE TABLE COMPANIES (ID SERIAL,
        "TYPE" VARCHAR(50),
        RUBY_TYPE VARCHAR(50),
        FIRM_ID INTEGER,
        NAME VARCHAR(50),
        CLIENT_OF INTEGER,
        RATING INTEGER default 1,
PRIMARY KEY (ID));

/* Table: TOPICS, Owner: SYSDBA */
CREATE TABLE TOPICS (ID INTEGER NOT NULL,
        TITLE VARCHAR(255),
        AUTHOR_NAME VARCHAR(255),
        AUTHOR_EMAIL_ADDRESS VARCHAR(255),
        WRITTEN_ON TIMESTAMP,
        LAST_READ DATE,
        CONTENT BLOB SUB_TYPE TEXT SEGMENT SIZE 80,
        REPLIES_COUNT INTEGER default 0,
        PARENT_ID INTEGER,
        "TYPE" VARCHAR(50),
        APPROVED SMALLINT DEFAULT 1,
PRIMARY KEY (ID));

SET TERM ^ ;

/* Triggers only will work for SQL triggers */
CREATE TRIGGER ACCOUNTS_INSERT FOR ACCOUNTS 
ACTIVE BEFORE INSERT POSITION 0 
as
begin
  if (new.id is null) then
    new.id = gen_id(gen_ACCOUNTS_id,1);
end ^
 
CREATE TRIGGER AUTO_ID_TESTS_INSERT FOR AUTO_ID_TESTS 
ACTIVE BEFORE INSERT POSITION 0 
as
begin
  if (new.auto_id is null) then
    new.auto_id = gen_id(gen_auto_id_tests_auto_id,1);
end ^
 
CREATE TRIGGER BOOLEANTESTS_INSERT FOR BOOLEANTESTS 
ACTIVE BEFORE INSERT POSITION 0 
as
begin
  if (new.id is null) then
    new.id = gen_id(gen_booleantests_id,1);
end ^
 
CREATE TRIGGER COLNAMETESTS_INSERT FOR COLNAMETESTS 
ACTIVE BEFORE INSERT POSITION 0 
as
begin
  if (new.id is null) then
    new.id = gen_id(gen_colnametests_id,1);
end ^
 
CREATE TRIGGER COMPANIES_INSERT FOR COMPANIES 
ACTIVE BEFORE INSERT POSITION 0 
as
begin
  if (new.id is null) then
    new.id = gen_id(gen_companies_id,1);
end ^
 
CREATE TRIGGER TOPICS_INSERT FOR TOPICS 
ACTIVE BEFORE INSERT POSITION 0 
as
begin
  if (new.id is null) then
    new.id = gen_id(gen_TOPICS_id,1);
end ^
 
COMMIT WORK ^
SET TERM ; ^

SET GENERATOR GEN_TOPICS_ID TO 100;
